#include <avr/io.h>
#include "CFAL12864C.h"

// all on PORTC
#define	LCD_DC  (1 << 7)
#define	LCD_RW  (1 << 6)	
#define	LCD_E   (1 << 5)	
#define	LCD_CS  (1 << 4)
#define	LCD_RES (1 << 2)

void bmp(bitmap_t b)
{
	int i,j;

	lcd_cmd(0x00);
	lcd_cmd(0x10);

	lcd_cmd(0xB0);

	for (j=0;j<8;j++)
	{
		for (i=0;i<128;i++)
		lcd_dat( b[j][i] );
		lcd_dat(0);
		lcd_dat(0);
		lcd_dat(0);
		lcd_dat(0);
	}

}
/*******************************************************************************************/
void delay(uint32_t twait)
{
	while (twait--)
		asm volatile ("nop");
}
/*******************************************************************************************/
void lcd_clr(uint8_t color)
{
	int i,p;

	lcd_cmd(0x40);

	for (p=0;p<8;p++) // pages
	{
		lcd_cmd(0xb0 + p);
		lcd_cmd(0x10);
		lcd_cmd(0x00);
		for (i=0;i<132;i++)	// ?????
		{
			lcd_dat(color);
		}
	}
}
/*******************************************************************************************/
void lcd_cmd(uint8_t cmd)
{
	PORTA = cmd;	// set up data on bus

	PORTC &= ~(LCD_DC | LCD_RW | LCD_CS);	// command mode, write mode, chip select

	// toggle E
	PORTC |= LCD_E;
	PORTC &= ~LCD_E;

	PORTC |= LCD_CS;	// remove chip select
}
/*******************************************************************************************/
void lcd_dat(uint8_t dat)
{
	PORTA = dat;	// set up data on bus

	PORTC |= LCD_DC;	// data mode
	PORTC &= ~(LCD_RW | LCD_CS);	// write mode, chip select

	// toggle E
	PORTC |= LCD_E;
	PORTC &= ~LCD_E;

	PORTC |= LCD_CS;	// remove chip select
}
/*******************************************************************************************/
void lcd_init(void)
{
	PORTC &= ~LCD_RES;
	delay(10000);
	PORTC |= LCD_RES;

	delay(20000);

	// enable VPP
	PORTD |= (1<<3);

	delay(200000L);

	lcd_cmd(0xa8); //Set Mux
	lcd_cmd(0x3f); //64mux

	lcd_cmd(0xa1); //Set Segment Re-map (0xa0 normal,0xa1 reverse)
	lcd_cmd(0xc0); //Set COM Output Scan Direction (0xc0 normal,0xc8 reverse)

	lcd_cmd(0x20); //Set Memory Addressing Mode
	lcd_cmd(0x00); //Horizontal

	lcd_cmd(0x81); //set contrast control
	lcd_cmd(0x40); //second byte

	lcd_cmd(0xaf); //Set Display ON

	delay(5000L);

	lcd_clr(0);
}
/*******************************************************************************************/
int main( void )
{
	PORTC = 0b11011110;
	PORTD = 0;		// all off

	DDRD |= (1<<3);	// VPP output
	DDRA = 0xFF;
	DDRC = 0xFE;
	DDRD |= 0x06;	// led pins output

	delay(20000L);

	lcd_init();

	while (1)
	{
		bmp(boot_logo);
		delay(5000000L);
		lcd_cmd(0xa6); //set inverse display
		bmp(boot_logo);
		delay(5000000L);
		lcd_cmd(0xa7); //set normal display 
	}

	return 0;
}
